<?php
/**
 * Exclude Product from coupon public section
 *
 * @link       
 * @since 2.0.1     
 *
 * @package  Wt_Smart_Coupon  
 */
if (!defined('ABSPATH')) {
    exit;
}
if(!class_exists('Wt_Smart_Coupon_Exclude_Product')) /* common module class not found so return */
{
	return;
}
if(!class_exists('Wt_Smart_Coupon_Exclude_Product_Public'))
{

	class Wt_Smart_Coupon_Exclude_Product_Public extends Wt_Smart_Coupon_Exclude_Product
    {
        public $module_base='exclude_product';
        public $module_id='';
        public static $module_id_static='';
        private static $instance = null;
        public function __construct()
        {
            $this->module_id=Wt_Smart_Coupon::get_module_id($this->module_base);
            self::$module_id_static=$this->module_id;

            add_filter('woocommerce_coupon_is_valid_for_product', array($this, 'set_coupon_validity_for_excluded_products'), 12, 4);
            add_filter('woocommerce_coupon_get_discount_amount', array($this, 'zero_discount_for_excluded_products'), 12, 5 );
        }

        /**
         * Get Instance
         * @since 2.0.1
         */
        public static function get_instance()
        {
            if(self::$instance==null)
            {
                self::$instance=new Wt_Smart_Coupon_Exclude_Product_Public();
            }
            return self::$instance;
        }

        /**
         * Check validity of coupon with exclude product.
         * @since 1.2.1
         * @since 2.0.2 [Bug fix] Coupons applying to the products from excluded category
         */
        public function set_coupon_validity_for_excluded_products($valid, $product, $coupon, $values)
        {
            if($this->check_if_excluded_product($product->get_id(), $coupon) === true )
            { 
                $valid = false;
            }
            //do not override valid param for WC default coupon types as WC itself validate the product and already determined whether it is valid or not
            elseif($coupon->is_type('store_credit')){
                $valid = true;
            } 

            return $valid;
        }

        /**
        * Check if the product is a disabled product
        *
        * @since  1.3.3
        * @since  1.3.4 Added checking with coupon excluded/included product list
        * @access public
        * @param  String Product id
        * @return bool
        */
        public function check_if_excluded_product($product_id, $coupon)
        {
            $excluded_products = $this->get_current_settings();
            
            $exclude_product = false; 
            
            if(!isset( $excluded_products['disabled_products']) || !is_array($excluded_products['disabled_products']) || empty( $excluded_products['disabled_products'])) {
                $exclude_product = false;
            }

            if($coupon->is_type('store_credit') )
            {
                /**
                 *  If this coupon is restricted to some products. Then check the current product is in the list
                 */
                $coupon_allowed_product_id_arr=$coupon->get_product_ids();            
                if(count($coupon_allowed_product_id_arr)>0 && !in_array($product_id, $coupon_allowed_product_id_arr))
                {
                    $exclude_product = true;
                }

                if(!$exclude_product)
                {
                    /**
                     *  If the current product is in the coupon excluded list
                     */
                    $coupon_excluded_product_id_arr=$coupon->get_excluded_product_ids();
                    if(in_array($product_id, $coupon_excluded_product_id_arr))
                    {
                        $exclude_product = true;
                    }
                }
                
                $disabled_products = ( isset( $excluded_products['disabled_store_credits'] ) && is_array( $excluded_products['disabled_store_credits'] ) ) ? $excluded_products['disabled_store_credits'] : array();
                
            }else
            {
                $disabled_products = ( isset( $excluded_products['disabled_products'] ) && is_array( $excluded_products['disabled_products'] ) ) ? $excluded_products['disabled_products'] : array();
            }

            $product = wc_get_product($product_id);
            $parent_id = $product->get_parent_id();

            if(in_array($product_id, $disabled_products) || (0 < $parent_id && in_array($parent_id, $disabled_products)))
            {
                $exclude_product = true;
            }
            
            return $exclude_product;
        }

        /**
        * Add zero discount for excluded products
        *
        * @since  1.3.3
        * @access public
        * @return float
        */
        public function zero_discount_for_excluded_products($discount, $discounting_amount, $cart_item, $single, $coupon)
        {          
            if(isset($cart_item['product_id']))
            {
                if($this->check_if_excluded_product($cart_item['product_id'], $coupon) === true)
                { 
                    $discount = 0;
                }
                   
            }     
            return $discount;
        }
    }
    Wt_Smart_Coupon_Exclude_Product_Public::get_instance();
}